// --------------------------------------------------------------------------------------
//
// GrandNapPat.CS                                                  Ulrich Mller 01.09.04
//                                                                 PCS1700       C# Final
// Grand Napoleon Patience : Logik, Legen und Anzeigen
// Basis PotiPatCS.DLL und QCard32.DLL
//
// Ablage   : obere Reihe mit den Pltzen 0 - 7 und Mittellcke in Breite von 3 Pltzen
// Auslage  : mittlerer Bereich mit 11 Leitern (0-10) mit max 40 Karten
// Keller   : hier nicht bentigt
// Talon    : unsichtbar, ber linker, oberer Ecke
//
// --------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;
using PotiPatCS;

namespace GrandNapCS {
	public class NapException : ApplicationException {
		public NapException() {}
		public NapException(string message): base(message) {}
		public NapException(string message, Exception inner) : base(message, inner) {}
	}

	public class GrandNapPat : PotiPatience {
		
		// --- Private Felder ------------------------------

		private KartenPaket Talon;
		private Ablage48    Ablage;
		private AuslageV    Auslage;
		
		private Card    SrcCard; 
		private CardPos SrcPos = new CardPos();
		private int[]   CardList = new int[40];

		private int[]   SuitList = {0, 3, 2, 1};

		private Panel pnlDisplay; Label lblRunde, lblRestKarten, lblRestTalon;
		private int SpielRunde, AnzZuege; bool BeimAuslegen;

		// --- Ereignisse -------------------------------------------------------------

		public delegate void WonDelegate(object sender, int Runde, int Zuege);
		public event    WonDelegate GameWon;

		// --- Konstruktor ------------------------------------------------------------

		public GrandNapPat(Form Canvas, Panel pnlDisplay, 
			Label lblRunde, Label lblRestKarten, Label lblRestTalon) : base(Canvas) {
			int XNull, XTalon, YNull, YAblage;

			this.lblRunde      = lblRunde;
			this.lblRestKarten = lblRestKarten;
			this.lblRestTalon  = lblRestTalon;
			this.pnlDisplay    = pnlDisplay;

			// --- Spielbereichskoordinaten ----------------------------
			XNull     = 16;
			YAblage   = 16;
			YNull     = YAblage + qc.CardHeight + 32;
			XTalon    = XNull + qc.CardWidthS * 9;

			// --- Spielbereichsobjekte --------------------------------
			Talon   = new KartenPaket(qc, 2, -qc.CardWidthS, -qc.CardHeigthS);
			Ablage  = new Ablage48(qc, 8, qc.CardWidthS, XNull, YAblage, 3);
			Auslage = new AuslageV(qc, 11, qc.CardWidthS, XNull, YNull, 
				Canvas.Height - YNull);

			// --- Anzeige-Labels ausrichten ----------------------------------------
			pnlDisplay.Top  = Ablage.Top + 5;
			pnlDisplay.Left = Ablage.Left + Ablage.Range.Width/2 - pnlDisplay.Width/2;
		}

		// --- Private Methoden -------------------------------------------------------

		// FindDestPos ----------------------------------------------------------------
		//             Bestimmen der angegebenen Postion in Pixelkoordinaten 
		// ----------------------------------------------------------------------------
		private CardPos FindDestPos(int X, int Y) {
			CardPos DestPos = new CardPos();

			DestPos.CardNr = 0;
			if(Ablage.Range.Contains(X, Y)){
				DestPos.Area = AreaCode.Ablage;
				DestPos.PosC = Ablage.XToCol(X);
			}
			else if (Auslage.Range.Contains(X, Y)) {
				DestPos.Area = AreaCode.Auslage;
				DestPos.PosC = Auslage.XToCol(X);
			}
			else {
				DestPos.Area = AreaCode.none;
				DestPos.PosC = 0;
			}
			DestPos.PosR = -1;
			return DestPos;
		}

		// --- Public Methoden --------------------------------------------------------

		// SpielStarten -------------------------------------------------------------
		//              Starten eines neuen Spieles : Initwerte, Mischen, Auslegen
		// --------------------------------------------------------------------------
		public override void SpielStarten(int SpielNr) {
			Graphics g = FCanvas.CreateGraphics();
			g.Clear(Color.Green);
			Talon.NewDeck(SpielNr);
			Ablage.Clear();
			Auslage.Clear();
			SpielRunde = 0;
			AnzZuege   = 0;
			NachsteRunde();
		}

		// Refresh ------------------------------------------------------------------
		//         Wiederherstellen des aktuellen Kartenbildes
		// --------------------------------------------------------------------------
		public override void Refresh() {
			Ablage.Paint();
			Auslage.Paint();
		}

		// StartZiehen ---------------------------------------------------------------
		//             Starten des Ziehens einer Karte wenn Maus ber Karte, sonst 
		//             merken der Mausposition
		// ---------------------------------------------------------------------------
		public override void StartZiehen(int X, int Y) {
			
			int SrcNr = qc.InitDrag(X, Y);  // --- Starten Drag, wenn Maus auf Karte
			
			if(SrcNr == 0) {                // --- Keine Karte
				SrcPos.CardNr = 0;
				SrcPos.PicX   = X;
				SrcPos.PicY   = Y;
				qc.AbortDrag();
				DragMode      = DragCode.none;
			}		
			else if(qc[SrcNr].Blocked) {    // --- Karte geblockt
				DragMode = DragCode.none;
				qc.AbortDrag();
			}
			else {                          // --- Karte liegt offen
				DragMode = DragCode.One;
				switch(qc[SrcNr].User2) {   // --- Auswahl Area
					case (int)AreaCode.Ablage:
						SrcCard = Ablage.TopMostCard(qc[SrcNr].User3);
						break;
					case (int)AreaCode.Auslage:
						SrcCard = Auslage.TopMostCard(qc[SrcNr].User3);
						break;
					default:
						break;
				}
			}
		}

		// WeiterZiehen ---------------------------------------------------------------
		//              Fortsetzen des Ziehens
		// ----------------------------------------------------------------------------
		public override void WeiterZiehen(int X, int Y) {
			switch((int)DragMode) {
				case (int)DragCode.One:
					qc.DoDrag(X, Y);
					break;
				default:
					break;
			}
		}

		// EndeZiehen ------------------------------------------------------------------
		//            Beenden des Ziehens, Bestimmen der Zielposition, Ablegen der Karte
		//            bzw. Rcklegen einer unpassenden Karte
		// -----------------------------------------------------------------------------
		public override void EndeZiehen(int X, int Y) {
			CardPos DestPos;
			if(SrcCard == null) return;
			int XM = qc[SrcCard.CardNr].X + 38;  // --- Koordinaten Kartenmitte
			int YM = qc[SrcCard.CardNr].Y + 46;

			switch((int)DragMode) {
				case (int)DragCode.One:   // --- Single Drag --------------
					qc.EndDrag(X, Y);
					DestPos = FindDestPos(XM, YM);
					switch((int)DestPos.Area) {
						case (int)AreaCode.Ablage:
							if((DestPos.PosC < 4 
								&& Ablage.IsAscendingSuit(SrcCard, DestPos.PosC)) 
								|| (DestPos.PosC > 3 
								&& Ablage.IsDescendingSuit(SrcCard, DestPos.PosC))) {
								Ablage.AddCard(Auslage.RemoveCard(SrcCard.PosC), 
									DestPos.PosC);
								AnzZuege++;
							}
							else qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY);
							break;
						case (int)AreaCode.Auslage:
							if(!BeimAuslegen && Auslage.IsUpDownSuit(SrcCard, 
								DestPos.PosC)) {
								Auslage.AddCard(Auslage.RemoveCard(SrcCard.PosC), 
									DestPos.PosC);
								AnzZuege++;
							}
							else qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY);
							break;
						default:
							qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY);
							break;
					}
					break;
				case (int)DragCode.Block:  // --- Block Drag --------------------
					break;
				default:
					break;
			}

			// --- Beenden Drag ---------------------------------------------------------
			DragMode = DragCode.none;
			lblRestKarten.Text = Convert.ToString(104 - Ablage.CardCount);
			if(Ablage.CardCount == 104) {
				if(GameWon != null) {
					GameWon(this, SpielRunde, AnzZuege);
				}
			}
		}

		// KarteAblegen ----------------------------------------------------------------
		//            Prfen, angegebene Karte ablegen (nach z.B. DoppelClick)
		// -----------------------------------------------------------------------------
		public override void KarteAblegen() {
			return;
		}

		// NachsteRunde ----------------------------------------------------------------
		//              Starten nchste von drei erlaubten Runden
		// -----------------------------------------------------------------------------
		public bool NachsteRunde() {
			SpielRunde++;
			lblRunde.Text = SpielRunde.ToString();
			if(SpielRunde > 1) {             // --- Karten einsammeln
				for(int i = 10; i >= 0; i--)
					while(!(Auslage.CardCount(i) == 0)) 
						Talon.AddCard(Auslage.RemoveCard(i));
			}
			// --- Auslegen Zeile 1 ------------------------------------------------------
			int Z = 0;
			while(Z <= 10 && Talon.CardCount > 0)
				Auslage.AddCard(Talon.RemoveCard(), Z++);
			BeimAuslegen = true;
			lblRestTalon.Visible = true;
			lblRestTalon.Text    = Talon.CardCount.ToString();
			return SpielRunde == 3 ? false : true;
		}

		// NachsteZeile ----------------------------------------------------------------
		//              Auslegen der nchsten Zeile vom (verdeckten) Talon,
		//              soweit Karten vorhanden.
		// -----------------------------------------------------------------------------
		public bool NachsteZeile() {
			Card tc; int i;
			// --- Nchste Zeile ------------------------------------------
			i = 0;
			while(i <= 10 && Talon.CardCount > 0) {
				tc = Auslage.TopMostCard(i);
				if(tc != null) tc.Disabled = true;
				Auslage.AddCard(Talon.RemoveCard(), i);
				if(i > 0 && i < 10) Auslage.TopMostCard(i).Disabled = true;
				i++;
			}
			// --- Disabled = false bei der letzten Zeile ----
			if(Talon.CardCount == 0) {
				for(i = 0; i <= 10; i++) Auslage.TopMostCard(i).Disabled = false;
				BeimAuslegen = false;
				lblRestTalon.Visible = false;
				return false;
			}
			BeimAuslegen = true;
			lblRestTalon.Text = Talon.CardCount.ToString();
			return true;
		}
	}
}
