// --------------------------------------------------------------------------------------
//
// Auslage.CS                                                      Ulrich Mller 26.06.04
//                                                                 PCS1700       C# Final
// Klassen fr die Auslage : AuslageV
// AuslageV : Auslage in Leitern (max. 13, mit jeweils max. 40 Karten, Blocking)
//
// --------------------------------------------------------------------------------------

using System;
using System.Drawing;

namespace PotiPatCS
{
	public class AuslageV {
		private QCard32Pool qc;
		private int[] actCard; int NrCols;
		private Card[,] Cards;
		private int XBasis, YBasis, XBreite, CWidth, XHoehe;

		public AuslageV(QCard32Pool QCard, int NrCols, int CardWidth, 
			int Left, int Top, int Height) {
			qc          = QCard;
			this.NrCols = NrCols;
			XBasis      = Left;
			YBasis      = Top;
			CWidth      = CardWidth;
			XBreite     = CWidth * NrCols - (CWidth - qc.CardWidth);
			XHoehe      = Height;
			actCard     = new int[NrCols];
			Cards       = new Card[NrCols, 40];
			Clear();
		}
		// --- Clear : Lschen der Auslage -------------------------------
		public void Clear() {
			for (int i = 0; i < NrCols; i++) actCard[i] = -1;
		}
		// --- CardCount : Anzahl Karten auf der angegebenen Spalte (0 = keine) ---
		public int CardCount(int PosC) {
			return actCard[PosC] + 1;
		}
		// --- Left / Top : Lesen der Koordinaten der linken, oberen Ecke ---
		public int Left{
			get{return XBasis;}
		}
		public int Top{
			get{return YBasis;}
		}
		// --- Range : Lesen der Bereichswerte von AuslageV -------------------
		public Rectangle Range{
			get{return new Rectangle(XBasis, YBasis, XBreite, XHoehe);}
		}
		// --- XToCol : Bestimmung der zu X gehrenden Spalte ------------------
		public int XToCol(int X) {
			return (int)((X - XBasis) / CWidth);
		}
		// --- TopMostCard : Auslesen der obersten Karte einer Spalte ----------
		public Card TopMostCard(int SrcCol) {
			return (actCard[SrcCol] < 0) ? null : Cards[SrcCol, actCard[SrcCol]];
		}
		// --- ActualCard : Auslesen der Karte auf der angegebenen Position ----
		public Card ActualCard(int PosC, int PosR) {
			return (actCard[PosC] < PosR) ? null : Cards[PosC, PosR];
		}
		// --- AddCard : Anlegen von SrcCard auf die angegebenen Spalte -------
		public void AddCard(Card SrcCard, int DestCol, FaceCode Face) {
			actCard[DestCol] += 1;
			Cards[DestCol, actCard[DestCol]] = SrcCard;

			SrcCard.PicX = DestCol * CWidth + XBasis;
			// --- verdeckte Karte 2 Pixel geschuppt, offene qc.Offest ablegen ---
			if(actCard[DestCol] > 0 && 
				Cards[DestCol, actCard[DestCol]-1].Face == FaceCode.FaceUp)
				SrcCard.PicY = Cards[DestCol, actCard[DestCol]-1].PicY + qc.Offset;
			else SrcCard.PicY = actCard[DestCol] * 2 + YBasis + 2;
			
			SrcCard.Area = AreaCode.Auslage;
			SrcCard.PosC = DestCol;
			SrcCard.PosR = actCard[DestCol];
			SrcCard.Face = Face;

			// --- Block unterliegenden Karten ---------------------------
			if(actCard[DestCol] > 0) Cards[DestCol, actCard[DestCol]-1].Blocked = true;

			qc.DealCard(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY);
			SrcCard.Blocked = false;  // --- unblock oberste Karte
		}
		public void AddCard(Card SrcCard, int DestCol){
			AddCard(SrcCard, DestCol, FaceCode.FaceUp);
		}
		// --- AddRange : Anlegen von Karten aus Tabelle -----------------------
		public void AddRange(Card[] CardList, int DestCol) {
			for(int i = 0; i < CardList.Length; i++) AddCard(CardList[i], DestCol);
		}
		// --- RemoveCard : Entfernen der obersten Karte der angegebenen Spalte ---
		public Card RemoveCard(int SrcCol) {
			if(actCard[SrcCol] < 0) return null;
			qc.RemoveCard(Cards[SrcCol, actCard[SrcCol]].CardNr);
			if(actCard[SrcCol] > 0) {
				Cards[SrcCol, actCard[SrcCol]-1].Disabled = false;
				Cards[SrcCol, actCard[SrcCol]-1].Blocked  = false;
				Cards[SrcCol, actCard[SrcCol]-1].Face     = FaceCode.FaceUp;
			}
			actCard[SrcCol] -= 1;
			return Cards[SrcCol, actCard[SrcCol]+1];
		}
		// --- RemoveRange : Entfernen der obersten Karten einer Spalte ---
		public Card[] RemoveRange(int SrcPos, int SrcCol) {
			int anzCard = CardCount(SrcCol) - SrcPos;
			Card[] cc = new Card[anzCard]; 
			for(int i = anzCard-1; i >= 0; i--) cc[i] = RemoveCard(SrcCol);
			return cc;
		}
		// Paint : Zeichnen des Kartenbildes -------------------------------
		public void Paint() {
			for(int i = 0; i < NrCols; i++)
				for(int j = 0; j <= actCard[i]; j++)
					qc.DrawCard(Cards[i, j].CardNr, Cards[i, j].PicX, Cards[i, j].PicY);
		}
		// --- IsDescendingChanging : Karte absteigend, wechselnde Farbe ------
		public bool IsDescendingChanging(Card SrcCard, int DestCol, ValueCode Start) {
			Card DestCard = TopMostCard(DestCol);

			if(DestCard == null) {
				if(Start == ValueCode.Alle) return true;
				return (SrcCard.Value == Start) ? true : false;
			}
			if(DestCard.Value == ValueCode.Ass) return false;
			return ((DestCard.Color != SrcCard.Color) && 
				(DestCard.Value == (SrcCard.Value + 1))) ? true : false;
		}
		public bool IsDescendingChanging(Card SrcCard, int DestCol) {
			return IsDescendingChanging(SrcCard, DestCol, ValueCode.Alle);
		}
		// --- IsUpDownSuit : Karte ab/aufsteigend in gleicher Farbe -----
		public bool IsUpDownSuit(Card SrcCard, int DestCol) {
			Card DestCard = TopMostCard(DestCol);
			if(DestCard == null) return true;
			int res = Math.Abs(DestCard.Value - SrcCard.Value);
			return ((DestCard.Suit == SrcCard.Suit) && ((res == 1) || (res == 12))) 
				? true : false;
		}
		// --- IsEnoughSpace : Gengend Platz zum Umlegen ------------------
		public bool IsEnoughSpace(int anzCard, int PosC) {
			int j = 0; int c = 0;
			for(int i = 0; i < NrCols; i++) if (actCard[i] < 0) {j += 1; c+= j;}
			if(actCard[PosC] >= 0) c += 1; else c -= 1;
			return c >= anzCard ? true : false;
		}
	}
}
