// --------------------------------------------------------------------------------------
//
// TestPatience.CS                                                 Ulrich Mller 27.06.04
//                                                                 PCS1700       C# Final
// TestPatience auf Basis der Klassenbibliothek PotiPatCS
//
// --------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;
using PotiPatCS;

public class TestPatException : ApplicationException {
	public TestPatException(){}
	public TestPatException(string message) : base(message){}
	public TestPatException(string message, Exception inner) : base(message, inner){}
}


namespace PotiPatTestCS
{
	public class TestPatience : PotiPatience
	{
		// --- Globale Daten -------------------------
		private KartenPaket Talon;
		private LiFoStack   Keller;
		private Ablage48    Ablage;
		private AuslageV    Auslage;

		private Card        SrcCard;
		private CardPos     SrcPos = new CardPos();
        private int[]       CardList = new int[20];
		private int         anzCard;

		private int[]       SuitList = {0, 3, 2, 1};

		public TestPatience(System.Windows.Forms.Form Canvas) : base(Canvas) {
			// --- Spielbereichsobjekte ------------------
			Talon   = new KartenPaket(qc, 1, 16, 16);
			Keller  = new LiFoStack(qc, 16 + qc.CardWidth + 12, 16);
			Ablage  = new Ablage48(qc, 4, qc.CardWidth + 12, 
				                   16 + ((qc.CardWidth + 12) * 3), 16);
			Auslage = new AuslageV(qc, 7, qc.CardWidth + 12, 
				                   16, 16 + qc.CardHeigthS, 500);
		}

		// --- FindDestPos : Bestimmen der logischen Position anhand der Pixellage
		public CardPos FindDestPos(int X, int Y) {
			CardPos DestPos = new CardPos();
			DestPos.CardNr = 0;
			if(Ablage.Range.Contains(X, Y)) {
				DestPos.Area = AreaCode.Ablage;
				DestPos.PosC = Ablage.XToCol(X);
			}
			else {if(Auslage.Range.Contains(X, Y)) {
					  DestPos.Area = AreaCode.Auslage;
					  DestPos.PosC = Auslage.XToCol(X);}
				  else {if(Keller.Range.Contains(X, Y)) {
							DestPos.Area = AreaCode.Keller;
							DestPos.PosC = 0;
						}
						else {
							DestPos.Area = AreaCode.none;
							DestPos.PosC = 0;
						}
				  }
			}
			DestPos.PosR = -1;
			return DestPos;
		}
		// --- CardToAblage : Prfen und Ablegen von Auslage oder Keller
		public bool CardToAblage(Card SrcCard) {
			int DestCol = SuitList[(int)SrcCard.Suit-1];
			if(Ablage.IsAscendingSuit(SrcCard, DestCol)) {
				if(SrcCard.Area == AreaCode.Auslage) {
					Ablage.AddCard(Auslage.RemoveCard(SrcCard.PosC), DestCol);
				}
				else {
					Ablage.AddCard(Keller.RemoveCard(), DestCol);
				}
				return true;
			}
			return false;
		}
		// --- SpielAuslegen ----------------------------------------
		private void SpielAuslegen(KartenPaket Talon) {
			Card SrcCard;

			Auslage.Clear();
			int OG = 0;
			for(int i = 0; i < 7; i++) {
				for(int j = 0; j <= OG; j++) {
					SrcCard = Talon.RemoveCard();
					if (j == OG) Auslage.AddCard(SrcCard, i, FaceCode.FaceUp);
					else {
						Auslage.AddCard(SrcCard, i, FaceCode.FaceDown);
						SrcCard.Disabled = true;
					}
				}
				OG++;
			}
		}
		// --- SpielStarten -----------------------------------------
		public override void SpielStarten(int SpielNr) {
			Graphics g = FCanvas.CreateGraphics();
			g.Clear(Color.Green);
			Talon.NewDeck(SpielNr);
			Keller.Clear();
			Ablage.Clear();
			SpielAuslegen(Talon);
		}
		// --- Refresh ---------------------------------------------
		public override void Refresh() {
			Talon.Paint();
			Keller.Paint();
			Ablage.Paint();
			Auslage.Paint();
		}
		// --- StartZiehen ----------------------------------------
		public override void StartZiehen(int X, int Y) {
			int SrcNr = qc.InitDrag(X, Y);
			if(SrcNr == 0) {
				SrcPos.CardNr = 0;
				SrcPos.PicX = X;
				SrcPos.PicY = Y;
				qc.AbortDrag();
				SrcCard = null;
				DragMode = DragCode.none;
				return;
			}
			SrcCard = new Card(SrcNr, qc[SrcNr].X, qc[SrcNr].Y);
			if(SrcCard.Blocked){
				DragMode = DragCode.Block;
				int j = 0;
				for(int i = SrcCard.PosR; i < Auslage.CardCount(SrcCard.PosC); i++)
					CardList[j++] = Auslage.ActualCard(SrcCard.PosC, i).CardNr;
				anzCard = Auslage.CardCount(SrcCard.PosC) - SrcCard.PosR;
			}
			else {
				DragMode = DragCode.One;
				if(SrcCard.Area == AreaCode.Talon) qc.AbortDrag();
			}
		}
		// --- WeiterZiehen -----------------------------------------
		public override void WeiterZiehen(int X, int Y) {
			switch(DragMode) {
				case DragCode.Block:
					qc.BlockDrag(CardList, anzCard, X, Y);
					break;
				case DragCode.One:
					qc.DoDrag(X, Y);
					break;
				default:
					break;
			}
		}
		// --- EndeZiehen -------------------------------------------
		public override void EndeZiehen(int X, int Y) {
			CardPos DestPos; int Nr;
			if(SrcCard == null) return;
			int XM = SrcCard.X + 38;
			int YM = SrcCard.Y + 46;
			switch (DragMode) {
				case DragCode.One:
					Nr = qc.EndDrag(X, Y);
					DestPos = FindDestPos(XM, YM);

				switch(DestPos.Area) {
					case AreaCode.Ablage:
						if(Ablage.IsAscendingSuit(SrcCard, DestPos.PosC)) {
							if(SrcCard.Area == AreaCode.Auslage) {
								Ablage.AddCard(Auslage.RemoveCard(SrcCard.PosC), 
									DestPos.PosC);
							}
							else {if(SrcCard.Area == AreaCode.Keller) {
									  Ablage.AddCard(Keller.RemoveCard(), DestPos.PosC);
								  }
								else qc.ReturnDrag(SrcCard.CardNr, 
										 SrcCard.PicX, SrcCard.PicY);
							}
						}
						else qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX,SrcCard.PicY);
						break;
					case AreaCode.Auslage:
						if(Auslage.IsDescendingChanging(SrcCard, DestPos.PosC, 
							ValueCode.Koenig)){
							if(SrcCard.Area == AreaCode.Auslage) {
								Auslage.AddCard(Auslage.RemoveCard(SrcCard.PosC), 
									DestPos.PosC);
							}
							else {if(SrcCard.Area == AreaCode.Keller){
									  Auslage.AddCard(Keller.RemoveCard(), 
										  DestPos.PosC, FaceCode.FaceUp);
								  }
								else qc.ReturnDrag(SrcCard.CardNr, 
										 SrcCard.PicX, SrcCard.PicY);
							}
						}
						else qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY);
						break;
					case AreaCode.Keller:
						if(SrcCard.Area == AreaCode.Talon){
							Keller.AddCard(Talon.RemoveCard());
						}
						else qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY);
						break;
				}
					break;
				case DragCode.Block:
					Nr = qc.EndBlockDrag(CardList, anzCard, X, Y);
					DestPos = FindDestPos(XM, YM);
					if(DestPos.Area == AreaCode.Auslage && 
						Auslage.IsDescendingChanging(SrcCard, 
						DestPos.PosC, ValueCode.Koenig)) 
						Auslage.AddRange(Auslage.RemoveRange(SrcCard.PosR, SrcCard.PosC), 
							DestPos.PosC);
					else qc.ReturnBlockDrag(CardList, anzCard, 
							 SrcCard.PicX, SrcCard.PicY);
					break;
				default:
					break;
			}
			DragMode = DragCode.none;
		}
		// --- KarteAblegen ----------------------------------------
		public override void KarteAblegen() {
			Card Karte;
			if(DragMode != DragCode.none) {
				qc.AbortDrag();
				DragMode = DragCode.none;
				switch(SrcCard.Area) {
					case AreaCode.Auslage:
					case AreaCode.Keller:
						CardToAblage(SrcCard);
						break;
					case AreaCode.Talon:
						Keller.AddCard(Talon.RemoveCard());
						break;
					default:
						break;
				}
			}
			else {
				if(Talon.Range.Contains(SrcPos.PicX, SrcPos.PicY)) {
					Karte = Talon.RemoveCard();
					while(Karte != null){
						Keller.AddCard(Karte);
						Karte = Talon.RemoveCard();
					}
					Karte = Keller.RemoveCard();
					while(Karte != null) {
						Talon.AddCard(Karte);
						Karte = Keller.RemoveCard();
					}
				}
			}
		}
	}
}
