// ------------------------------------------------------------------------------------
//
// QCard32Wrapper.CS                                             Ulrich Mller 23.06.04
//                                                               PCS1700       C# Final
//
// QCard32 Kartenpool auf Basis der QCard32.DLL von Stephen Murphy
// www.telusplanet.net/public/stevem
// Der Kartenpool umfat zwei Spiele a 52 Karten, die von 1 - 104 durchnumeriert sind
// Zu jeder Karte sind statische Werte (Wert, Farbe ...) und dynamische Werte
// (aktuelle Position, FaceUp, Blocked, Disabled ...) ber ihre Kartennr. verfgbar.
// Hinzukommen Methoden zur Verwaltung des Kartenpools und zur Handhabung einzelner
// Karten oder auch von Kartenfolgen (BlockDrag)
// Untersttzt wird das Postionieren und Zeichnen der Karten auf einer angebbaren
// (hwnd) Zeichenflche, beim Positionieren wird die aktuelle Position nachgehalten.
//
// -------------------------------------------------------------------------------------

using System;
using System.Runtime.InteropServices;

namespace PotiPatCS
{	
	public enum SuitCode {
		Karo  = 2, // --- Diamonds
		Herz  = 3, // --- Hearts
		Pik   = 4, // --- Spades
		Kreuz = 1  // --- Clubs
	}
	public enum ColorCode {
		Schwarz = 1,
		Rot     = 2
	}
	public enum ValueCode {
		Alle   = 0,	 Ass    = 1,  Zwei   = 2,  Drei   = 3,	Vier   = 4,
		Fuenf  = 5,	 Sechs  = 6,  Sieben = 7,  Acht   = 8,	Neun   = 9,
		Zehn   = 10, Bube   = 11, Dame   = 12, Koenig = 13
	}
	public enum FaceCode {
		FaceDown = 0, FaceUp = 1
	}

	public class QCard32Exception: ApplicationException	{
		public QCard32Exception(){}
		public QCard32Exception(string message) : base(message){}
		public QCard32Exception(string message, Exception inner) : base(message, inner){}
	}

	public class QCard32Pool
	{
		// --- QCard32.DLL Deklarationen ----------------------------
		
		[DllImport("qcard32.dll", EntryPoint = "InitializeDeck")]
		private   static extern int  InitializeDeckQC(IntPtr hwnd);
		[DllImport("qcard32.dll", EntryPoint = "SetDefaultValues")]
		internal static extern void SetDefaultValuesQC();
		[DllImport("qcard32.dll", EntryPoint = "SetCurrentBack")]
		internal static extern void SetCurrentBackQC(int nIndex);

		// --- Drawing Funktionen -----------------------------------
		[DllImport("qcard32.dll", EntryPoint = "DrawSymbol")]
		private static extern void DrawSymbolQC(IntPtr hwnd, int nValue, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "DrawCard")]
		private static extern void DrawCardQC(IntPtr hwnd, int nCard, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "DrawBack")]
		private static extern void DrawBackQC(IntPtr hwnd, int nValue, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "DealCard")]
		private static extern void DealCardQC(IntPtr hwnd, int nCard, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "RemoveCard")]
		private static extern void RemoveCardQC(IntPtr hwnd, int nCard);

		// --- Informations GetFunktionen --------------------------------
		[DllImport("qcard32.dll", EntryPoint = "GetCardColor")]
		internal static extern int GetCardColorQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetCardSuit")]
		internal static extern int GetCardSuitQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetCardValue")]
		internal static extern int GetCardValueQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetCardStatus")]
		internal static extern int GetCardStatusQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetCardBlocked")]
		internal static extern int GetCardBlockedQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "IsCardDisabled")]
		internal static extern int IsCardDisabledQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetCardX")]
		internal static extern int GetCardXQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetCardY")]
		internal static extern int GetCardYQC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetUser1")]
		internal static extern int GetUser1QC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetUser2")]
		internal static extern int GetUser2QC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetUser3")]
		internal static extern int GetUser3QC(int nCard);
		[DllImport("qcard32.dll", EntryPoint = "GetUser4")]
		internal static extern int GetUser4QC(int nCard);

		// --- Informations SetFunktionen --------------
		[DllImport("qcard32.dll", EntryPoint = "SetCardStatus")]
		internal static extern void SetCardStatusQC(int nCard, int bValue);
		[DllImport("qcard32.dll", EntryPoint = "AdjustCardBlocked")]
		internal static extern void AdjustCardBlockedQC(int nCard, int bValue);
		[DllImport("qcard32.dll", EntryPoint = "SetCardDisabled")]
		internal static extern void SetCardDisabledQC(int nCard, int bValue);
		[DllImport("qcard32.dll", EntryPoint = "SetCardX")]
		internal static extern void SetCardXQC(int nCard, int x);
		[DllImport("qcard32.dll", EntryPoint = "SetCardY")]
		internal static extern void SetCardYQC(int nCard, int y);
		[DllImport("qcard32.dll", EntryPoint = "SetUser1")]
		internal static extern void SetUser1QC(int nCard, int bValue);
		[DllImport("qcard32.dll", EntryPoint = "SetUser2")]
		internal static extern void SetUser2QC(int nCard, int bValue);
		[DllImport("qcard32.dll", EntryPoint = "SetUser3")]
		internal static extern void SetUser3QC(int nCard, int bValue);
		[DllImport("qcard32.dll", EntryPoint = "SetUser4")]
		internal static extern void SetUser4QC(int nCard, int bValue);
		[DllImport("qcard32.dll", EntryPoint = "SetOffSet")]
		internal static extern void SetOffSetQC(int nValue);

		// --- Dragging Funktionen ---------------------------------
		[DllImport("qcard32.dll", EntryPoint = "InitDrag")]
		private static extern int InitDragQC(IntPtr hwnd, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "AbortDrag")]
		private static extern void AbortDragQC();
		[DllImport("qcard32.dll", EntryPoint = "DoDrag")]
		private  static extern void DoDragQC(IntPtr hwnd, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "BlockDrag")]
		private static extern void BlockDragQC(IntPtr hwnd, ref int nFirst,
			int nNumCards, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "EndDrag")]
		private static extern int EndDragQC(IntPtr hwnd, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "EndBlockDrag")]
		private static extern int EndBlockDragQC(IntPtr hwnd, ref int nFirst,
			int nNumCards, int x, int y);
		[DllImport("qcard32.dll", EntryPoint = "ReturnDrag")]
		private static extern void ReturnDragQC(IntPtr hwnd, int nCard, 
			int nOldX, int OldY);
		[DllImport("qcard32.dll", EntryPoint = "ReturnBlockDrag")]
		private static extern void ReturnBlockDragQC(IntPtr hwnd, ref int nFirst,
			int nNumCards, int x, int y);

		// --- Undokumentiert -------------------------------------------
		[DllImport("qcard32.dll")]
		private static extern int GetFreeDestination(int nSource);
		[DllImport("qcard32.dll")]
		private static extern int PointInFreeCard(int x, int y);
		[DllImport("qcard32.dll")]
		private static extern int PointInCardTop(int x, int y);
		[DllImport("qcard32.dll")]
		private static extern void SetActiveCard(int nCard);

		// --- Globale Daten -------
		private static bool FirstTime = true;
		private IntPtr PaintHandle;
		private int FBackNr;
		private int FCardWidth;
		private int FCardWidthS;
		private int FCardHeight;
		private int FCardHeightS;
		private int FOffset;

		// --- Konstruktor -----------------
		public QCard32Pool(IntPtr Handle, int BackNr, int Offset) {
			if(InitializeDeckQC(Handle) == 0) 
				throw new QCard32Exception("LadeProblem bei QCard32.DLL");
			if (!FirstTime)
				throw new QCard32Exception("Nur eine Instanz von QCard32Pool zulssig");
			PaintHandle = Handle;
			FirstTime = false;
			FBackNr = BackNr;
			SetCurrentBackQC(FBackNr);
			FCardWidth = 71;
			FCardWidthS = FCardWidth + 3;
			FCardHeight = 96;
			FCardHeightS = FCardHeight + 3;
			FOffset = Offset;
			SetOffSetQC(FOffset);
		}
		public QCard32Pool(IntPtr Handle) : this(Handle, 1, 20) {}

		// --- Pool-Eigenschaften -------------------------------------------------
		public int BackNr {
			get{return FBackNr;}
			set{FBackNr = value; SetCurrentBackQC(FBackNr);}
		}
		public int CardWidth {
			get{return FCardWidth;}
		}
		public int CardWidthS {
			get{return FCardWidthS;}
			set{FCardWidthS = value <= FCardWidth ? FCardWidth : value;}
		}
		public int CardHeight {
			get{return FCardHeight;}
		}
		public int CardHeigthS {
			get{return FCardHeightS;}
			set{FCardHeightS = value <= FCardHeight ? FCardHeight : value;}
		}
		public int Offset {
			get{return FOffset;}
			set{FOffset = value; SetOffSetQC(FOffset);}
		}

		// --- Drawing Methoden ----------------------		
		public void DrawSymbol(int nValue, int x, int y) {
			DrawSymbolQC(PaintHandle, nValue, x, y);
		}
		public void DrawCard(int nCard, int x, int y) {
			DrawCardQC(PaintHandle, nCard, x, y);
		}
		public void DrawBack(int nValue, int x, int y) {
			DrawBackQC(PaintHandle, nValue, x, y);
		}
		public void DealCard(int nCard, int x, int y) {
			DealCardQC(PaintHandle, nCard, x, y);
		}
		public void RemoveCard(int nCard) {
			RemoveCardQC(PaintHandle, nCard);
		}

		// --- Dragging Methoden -----------------------------------
		public int InitDrag(int x, int y) {
			return InitDragQC(PaintHandle, x, y);
		}
		public void AbortDrag() {
			AbortDragQC();
		}
		public void DoDrag(int x, int y) {
			DoDragQC(PaintHandle, x, y);
		}
		public void BlockDrag(int[] nCardList, int nNumCards, int x, int y) {
			BlockDragQC(PaintHandle, ref nCardList[0], nNumCards, x, y);
		}
		public int EndDrag(int x, int y) {
			return EndDragQC(PaintHandle, x, y);
		}
		public int EndBlockDrag(int[] nCardList, int nNumCards, int x, int y) {
			return EndBlockDragQC(PaintHandle, ref nCardList[0], nNumCards, x, y);
		}
		public void ReturnDrag(int nCard, int nOldX, int nOldY) {
			ReturnDragQC(PaintHandle, nCard, nOldX, nOldY);
		}
		public void ReturnBlockDrag(int[] nCardList, int nNumCards, int x, int y) {
			ReturnBlockDragQC(PaintHandle, ref nCardList[0], nNumCards, x, y);
		}

		// --- Indexer ber Karten-Eigenschaften -----
		public PoolCard this[int index]{
			get{return new PoolCard(index);}
			set{;}
		}
	}
}